@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2014 Andrew Ziem
+# Copyright (C) 2008-2015 Andrew Ziem
 # http://bleachbit.sourceforge.net
 #
 # This program is free software: you can redistribute it and/or modify
@@ -84,12 +84,20 @@ class FileActionProvider(ActionProvider):
         """Initialize file search"""
         self.regex = action_element.getAttribute('regex')
         assert(isinstance(self.regex, (str, unicode, types.NoneType)))
+        self.nregex = action_element.getAttribute('nregex')
+        assert(isinstance(self.nregex, (str, unicode, types.NoneType)))
         self.search = action_element.getAttribute('search')
         self.path = os.path.expanduser(os.path.expandvars(
             action_element.getAttribute('path')))
+        if 'nt' == os.name and self.path:
+            # convert forward slash to backslash for compatibility with getsize()
+            # and for display.  Do not convert an empty path, or it will become
+            # the current directory (.).
+            self.path = os.path.normpath(self.path)
         self.ds = {}
         if 'deep' == self.search:
             self.ds['regex'] = self.regex
+            self.ds['nregex'] = self.nregex
             self.ds['cache'] = General.boolstr_to_bool(
                 action_element.getAttribute('cache'))
             self.ds['command'] = action_element.getAttribute('command')
@@ -130,13 +138,35 @@ class FileActionProvider(ActionProvider):
         else:
             raise RuntimeError("invalid search='%s'" % self.search)
 
-        if None == self.regex:
+        if self.regex:
+            regex_c = re.compile(self.regex)
+
+        if self.nregex:
+            nregex_c = re.compile(self.nregex)
+
+        # Sometimes this loop repeats many times, so optimize it by
+        # putting the conditional outside the loop.
+
+        if not self.regex and not self.nregex:
             for path in func(self.path):
                 yield path
+        elif self.regex and not self.nregex:
+            for path in func(self.path):
+                if not regex_c.search(os.path.basename(path)):
+                    continue
+                yield path
+        elif self.nregex:
+            for path in func(self.path):
+                if nregex_c.search(os.path.basename(path)):
+                    continue
+                yield path
         else:
             for path in func(self.path):
-                if re.search(self.regex, os.path.basename(path)):
-                    yield path
+                if not regex_c.search(os.path.basename(path)):
+                    continue
+                if nregex_c.search(os.path.basename(path)):
+                    continue
+                yield path
 
     def get_commands(self):
         raise NotImplementedError('not implemented')
@@ -345,57 +375,17 @@ class Truncate(FileActionProvider):
             yield Command.Truncate(path)
 
 
-class TestActionProvider(ActionProvider):
+class WinShellChangeNotify(ActionProvider):
 
-    """Test ActionProvider"""
-    action_key = 'test'
-
-    def __init__(self, action_element):
-        self.pathname = action_element.getAttribute('path')
+    """Action to clean the Windows Registry"""
+    action_key = 'win.shell.change.notify'
 
     def get_commands(self):
-        # non-existent file, should fail and continue
-        yield Command.Delete("doesnotexist")
-
-        # access denied, should fail and continue
-        def accessdenied():
-            raise OSError(13, 'Permission denied: /foo/bar')
-
-        yield Command.Function(None, accessdenied, 'Test access denied')
-        # Lock the file on Windows.  It should be marked for deletion.
-        if 'nt' == os.name:
-            f = os.open(self.pathname, os.O_RDWR | os.O_EXCL)
-            yield Command.Delete(self.pathname)
-            assert(os.path.exists(self.pathname))
-            os.close(f)
-
-        # function with path, should succeed
-        def pathfunc(path):
-            pass
-        # self.pathname must exist because it checks the file size
-        yield Command.Function(self.pathname, pathfunc, 'pathfunc')
-
-        # function generator without path, should succeed
-        def funcgenerator():
-            yield long(10)
-        yield Command.Function(None, funcgenerator, 'funcgenerator')
-
-        # plain function without path, should succeed
-        def intfunc():
-            return int(10)
-        yield Command.Function(None, intfunc, 'intfunc')
-
-        # truncate real file
-        yield Command.Truncate(self.pathname)
-
-        # real file, should succeed
-        yield Command.Delete(self.pathname)
-
-        # file with invalid encoding
-        import tempfile
-        (fd, filename) = tempfile.mkstemp('invalid-encoding-\xe4\xf6\xfc~')
-        os.close(fd)
-        yield Command.Delete(filename)
+        import Windows
+        yield Command.Function(
+            None,
+            Windows.shell_change_notify,
+            None)
 
 
 class Winreg(ActionProvider):
