@@ -19,11 +19,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+import logging
 import os
 import sys
 import threading
 import time
-import traceback
 import types
 import warnings
 
@@ -62,7 +62,7 @@ class TreeInfoModel:
 
     def __init__(self):
         self.tree_store = gtk.TreeStore(
-            gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_PYOBJECT)
+            gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
         if None == self.tree_store:
             raise Exception("cannot create tree store")
         self.row_changed_handler_id = None
@@ -89,17 +89,22 @@ class TreeInfoModel:
             self.tree_store.disconnect(self.row_changed_handler_id)
         self.tree_store.clear()
         for key in sorted(backends):
+            if not any(backends[key].get_options()):
+                # localizations has no options, so it should be hidden
+                # https://github.com/az0/bleachbit/issues/110
+                continue
             c_name = backends[key].get_name()
             c_id = backends[key].get_id()
             c_value = options.get_tree(c_id, None)
             if not c_value and options.get('auto_hide') \
                     and backends[key].auto_hide():
-                print "info: automatically hiding cleaner '%s'" % (c_id)
+                logger = logging.getLogger(__name__)
+                logger.debug("automatically hiding cleaner '%s'", (c_id))
                 continue
-            parent = self.tree_store.append(None, (c_name, c_value, c_id))
+            parent = self.tree_store.append(None, (c_name, c_value, c_id, ""))
             for (o_id, o_name) in backends[key].get_options():
                 o_value = options.get_tree(c_id, o_id)
-                self.tree_store.append(parent, (o_name, o_value, o_id))
+                self.tree_store.append(parent, (o_name, o_value, o_id, ""))
         self.row_changed_handler_id = self.tree_store.connect("row-changed",
                                                               self.on_row_changed)
 
@@ -139,6 +144,18 @@ class TreeDisplayModel:
         self.column1.add_attribute(self.renderer1, "active", 1)
         self.view.append_column(self.column1)
 
+        # third column
+        self.renderer2 = gtk.CellRendererText()
+        if hasattr(self.renderer2, 'set_alignment'):
+            # requires PyGTK 2.22
+            # http://www.pygtk.org/pygtk2reference/class-gtkcellrenderer.html#method-gtkcellrenderer--set-alignment
+            self.renderer2.set_alignment(1.0, 0.0)
+        # TRANSLATORS: Size is the label for the column that shows how
+        # much space an option would clean or did clean
+        self.column2 = gtk.TreeViewColumn(_("Size"), self.renderer2, text=3)
+        self.column2.set_alignment(1.0)
+        self.view.append_column(self.column2)
+
         # finish
         self.view.expand_all()
         return self.view
@@ -205,6 +222,25 @@ class TreeDisplayModel:
         return
 
 
+class GtkLoggerHandler(logging.Handler):
+
+    def __init__(self, append_text):
+        logging.Handler.__init__(self)
+        self.append_text = append_text
+        self.min_level = logging.WARNING
+        if '--debug-log' in sys.argv:
+            self.min_level = logging.DEBUG
+
+    def emit(self, record):
+        if record.levelno < self.min_level:
+            return
+        tag = 'error' if record.levelno >= logging.WARNING else None
+        msg = record.getMessage()
+        if record.exc_text:
+            msg = msg + '\n' + record.exc_text
+        self.append_text(msg + '\n', tag)
+
+
 class GUI:
 
     """The main application GUI"""
@@ -341,10 +377,9 @@ class GUI:
             self.textbuffer.set_text("")
             self.progressbar.show()
             self.worker = Worker.Worker(self, really_delete, operations)
-        except:
-            traceback.print_exc()
-            err = str(sys.exc_info()[1])
-            self.append_text(err + "\n", 'error')
+        except Exception, e:
+            logger = logging.getLogger(__name__)
+            logger.exception('Error in Worker()')
         else:
             self.start_time = time.time()
             worker = self.worker.run()
@@ -367,13 +402,14 @@ class GUI:
 
         # notification for long-running process
         elapsed = (time.time() - self.start_time)
-        print 'debug: elapsed time: %d seconds' % elapsed
+        logger = logging.getLogger(__name__)
+        logger.debug('elapsed time: %d seconds', elapsed)
         if elapsed < 10 or self.window.is_active():
             return
         try:
             import pynotify
         except:
-            print "debug: pynotify not available"
+            logger.debug('pynotify not available')
         else:
             if pynotify.init(APP_NAME):
                 notify = pynotify.Notification('BleachBit', _("Done."),
@@ -534,7 +570,8 @@ class GUI:
 
         # prompt the user to confirm
         if not self.shred_paths(paths):
-            print 'user aborted'
+            logger = logging.getLogger(__name__)
+            logger.debug('user aborted shred')
             # aborted
             return
 
@@ -610,6 +647,27 @@ class GUI:
         else:
             raise RuntimeError('unexpected type: ' + str(type(status)))
 
+    def update_item_size(self, option, option_id, bytes_removed):
+        """Update size in tree control"""
+        model = self.view.get_model()
+
+        text = FileUtilities.bytes_to_human(bytes_removed)
+        if 0 == bytes_removed:
+            text = ""
+
+        __iter = model.get_iter_root()
+        while __iter:
+            if model[__iter][2] == option:
+                if (option_id == -1):
+                    model[__iter][3] = text
+                else:
+                    child = model.iter_children(__iter)
+                    while child:
+                        if model[child][2] == option_id:
+                            model[child][3] = text
+                        child = model.iter_next(child)
+            __iter = model.iter_next(__iter)
+
     def update_total_size(self, bytes_removed):
         """Callback to update the total size cleaned"""
         context_id = self.status_bar.get_context_id('size')
@@ -788,10 +846,9 @@ class GUI:
             if updates:
                 gobject.idle_add(
                     lambda: Update.update_dialog(self.window, updates))
-        except:
-            traceback.print_exc()
-            self.append_text(
-                _("Error when checking for updates: ") + str(sys.exc_info()[1]), 'error')
+        except Exception, e:
+            logger = logging.getLogger(__name__)
+            logger.exception(_("Error when checking for updates: "))
 
     def __init__(self, uac=True, shred_paths=None):
         if uac and 'nt' == os.name and Windows.elevate_privileges():
@@ -802,6 +859,17 @@ class GUI:
         register_cleaners()
         self.create_window()
         gobject.threads_init()
+
+        # Redirect logging to the GUI.
+        bb_logger = logging.getLogger('bleachbit')
+        gtklog = GtkLoggerHandler(self.append_text)
+        bb_logger.addHandler(gtklog)
+        if 'nt' == os.name and 'windows_exe' == getattr(sys, 'frozen', None):
+            # On Microsoft Windows this avoids py2exe redirecting stderr to
+            # bleachbit.exe.log.
+            # sys.frozen = console_exe means the console is shown
+            from Common import logger_sh
+            bb_logger.removeHandler(logger_sh)
         if shred_paths:
             self.shred_paths(shred_paths)
             return
@@ -818,10 +886,9 @@ class GUI:
             try:
                 import sqlite3
             except ImportError, e:
-                print e
-                print dir(e)
-                self.append_text(
-                    _("Error loading the SQLite module: the antivirus software may be blocking it."), 'error')
+                logger = logging.getLogger(__name__)
+                logger.exception(
+                    _("Error loading the SQLite module: the antivirus software may be blocking it."))
         if 'posix' == os.name and os.path.expanduser('~') == '/root':
             self.append_text(
                 _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.'))
