@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2014 Andrew Ziem
+# Copyright (C) 2008-2015 Andrew Ziem
 # http://bleachbit.sourceforge.net
 #
 # This program is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@ Check for updates via the Internet
 
 
 import hashlib
+import logging
 import os
 import os.path
 import platform
@@ -54,12 +55,16 @@ def update_winapp2(url, hash_expected, append_text, cb_success):
     # download update
     opener = urllib2.build_opener()
     opener.addheaders = [('User-Agent', user_agent())]
-    doc = opener.open(url).read()
+    kwargs = {'fullurl': url}
+    if sys.hexversion >= 0x02060000:
+        # Python 2.6 added timeout option
+        kwargs['timeout'] = 20
+    doc = opener.open(**kwargs).read()
     # verify hash
     hash_actual = hashlib.sha512(doc).hexdigest()
     if hash_expected and not hash_actual == hash_expected:
         raise RuntimeError("hash for %s actually %s instead of %s" %
-                          (url, hash_actual, hash_expected))
+                           (url, hash_actual, hash_expected))
     # delete current
     if delete_current:
         from FileUtilities import delete
@@ -97,10 +102,17 @@ def user_agent():
         import locale
         __locale = locale.getdefaultlocale()[0]  # e.g., en_US
     except:
-        traceback.print_exc()
+        logger = logging.getLogger(__name__)
+        logger.exception('Exception when getting default locale')
 
-    agent = "BleachBit/%s (%s; %s; %s)" % (Common.APP_VERSION,
-                                           __platform, __os, __locale)
+    try:
+        import gtk
+        gtkver = '; GTK %s' % '.'.join([str(x) for x in gtk.gtk_version])
+    except:
+        gtkver = ""
+
+    agent = "BleachBit/%s (%s; %s; %s%s)" % (Common.APP_VERSION,
+                                             __platform, __os, __locale, gtkver)
     return agent
 
 
@@ -143,13 +155,20 @@ def check_updates(check_beta, check_winapp2, append_text, cb_success):
     opener = urllib2.build_opener()
     socket.setdefaulttimeout(Common.socket_timeout)
     opener.addheaders = [('User-Agent', user_agent())]
-    handle = opener.open(Common.update_check_url)
+    logger = logging.getLogger(__name__)
+    try:
+        handle = opener.open(Common.update_check_url)
+    except urllib2.URLError:
+        logger.exception(
+            _('Error when opening a network connection to %s to check for updates. Please verify the network is working.' %
+                Common.update_check_url))
+        return ()
     doc = handle.read()
     try:
         dom = xml.dom.minidom.parseString(doc)
     except:
-        print doc
-        raise
+        logger.exception('The update information does not parse: %s' % doc)
+        return ()
 
     def parse_updates(element):
         if element:
