@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2014 Andrew Ziem
+# Copyright (C) 2008-2015 Andrew Ziem
 # http://bleachbit.sourceforge.net
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,6 +23,7 @@ Perform the preview or delete operations
 """
 
 
+import logging
 import math
 import os
 import sys
@@ -45,6 +46,7 @@ class Worker:
             append_text()
             update_progress_bar()
             update_total_size()
+            update_item_size()
             worker_done()
         really_delete: (boolean) preview or make real changes?
         operations: dictionary where operation-id is the key and
@@ -54,6 +56,7 @@ class Worker:
         self.really_delete = really_delete
         assert(isinstance(operations, dict))
         self.operations = operations
+        self.size = 0
         self.total_bytes = 0
         self.total_deleted = 0
         self.total_errors = 0
@@ -70,9 +73,8 @@ class Worker:
         # replaced by a message such as 'Permission denied.'
         err = _("Exception while running operation '%(operation)s': '%(msg)s'") \
             % {'operation': operation, 'msg': str(sys.exc_info()[1])}
-        print err
-        traceback.print_exc()
-        self.ui.append_text(err + "\n", 'error')
+        logger = logging.getLogger(__name__)
+        logger.error(err, exc_info=True)
         self.total_errors += 1
 
     def execute(self, cmd):
@@ -91,19 +93,25 @@ class Worker:
             # 2 = does not exist
             # 13 = permission denied
             from errno import ENOENT, EACCES
-            if not (isinstance(e, OSError) and e.errno in (ENOENT, EACCES)):
-                traceback.print_exc()
-            line = "%s\n" % (str(sys.exc_info()[1]))
+            logger = logging.getLogger(__name__)
+            if (isinstance(e, OSError) and e.errno in (ENOENT, EACCES)):
+                # For access denied, do not show traceback
+                logger.error('%s: %s' % (str(e), str(cmd)))
+            else:
+                # For other errors, show the traceback.
+                logger.error('Error in execution of %s' %
+                             str(cmd), exc_info=True)
             self.total_errors += 1
-            self.ui.append_text(line, 'error')
         else:
             if None == ret:
                 return
             if isinstance(ret['size'], (int, long)):
                 size = FileUtilities.bytes_to_human(ret['size'])
+                self.size += ret['size']
                 self.total_bytes += ret['size']
             else:
                 size = "?B"
+
             if ret['path']:
                 path = ret['path']
             else:
@@ -112,13 +120,18 @@ class Worker:
             line = u"%s %s %s\n" % (ret['label'], size, path)
             self.total_deleted += ret['n_deleted']
             self.total_special += ret['n_special']
-            self.ui.append_text(line)
+            if ret['label']:
+                # the label may be a hidden operation
+                # (e.g., win.shell.change.notify)
+                self.ui.append_text(line)
 
     def clean_operation(self, operation):
         """Perform a single cleaning operation"""
         operation_options = self.operations[operation]
         assert(isinstance(operation_options, list))
-        print "debug: clean_operation('%s'), options = '%s'" % (operation, operation_options)
+        logger = logging.getLogger(__name__)
+        logger.debug("clean_operation('%s'), options = '%s'" %
+                     (operation, operation_options))
 
         if not operation_options:
             raise StopIteration
@@ -133,7 +146,9 @@ class Worker:
         import time
         self.yield_time = time.time()
 
+        total_size = 0
         for option_id in operation_options:
+            self.size = 0
             assert(isinstance(option_id, (str, unicode)))
             # normal scan
             for cmd in backends[operation].get_commands(option_id):
@@ -148,6 +163,10 @@ class Worker:
                         self.ui.update_total_size(self.total_bytes)
                     yield True
                     self.yield_time = time.time()
+
+            self.ui.update_item_size(operation, option_id, self.size)
+            total_size += self.size
+
             # deep scan
             for ds in backends[operation].get_deep_scan(option_id):
                 if '' == ds['path']:
@@ -158,6 +177,7 @@ class Worker:
                 if not self.deepscans.has_key(ds['path']):
                     self.deepscans[ds['path']] = []
                 self.deepscans[ds['path']].append(ds)
+        self.ui.update_item_size(operation, -1, total_size)
 
     def run_delayed_op(self, operation, option_id):
         """Run one delayed operation"""
@@ -241,6 +261,7 @@ class Worker:
 
         # print final stats
         bytes_delete = FileUtilities.bytes_to_human(self.total_bytes)
+
         if self.really_delete:
             # TRANSLATORS: This refers to disk space that was
             # really recovered (in other words, not a preview)
@@ -274,7 +295,8 @@ class Worker:
 
     def run_deep_scan(self):
         """Run deep scans"""
-        print 'debug: deepscans=', self.deepscans
+        logger = logging.getLogger(__name__)
+        logger.debug(' deepscans=%s' % self.deepscans)
         # TRANSLATORS: The "deep scan" feature searches over broad
         # areas of the file system such as the user's whole home directory
         # or all the system executables.
@@ -282,8 +304,7 @@ class Worker:
         yield True  # allow GTK to update the screen
         ds = DeepScan.DeepScan()
         for (path, dsdict) in self.deepscans.iteritems():
-            print 'debug: deepscan path=', path
-            print 'debug: deepscan dict=', dsdict
+            logger.debug('deepscan path=%s, dict=%s' % (path, dsdict))
             for dsdict2 in dsdict:
                 ds.add_search(path, dsdict2['regex'])
 
