@@ -23,6 +23,7 @@ Test cases for module Action
 """
 
 
+import shutil
 import sys
 import tempfile
 import unittest
@@ -34,6 +35,46 @@ from bleachbit.Action import *
 import common
 
 
+def _action_str_to_commands(action_str):
+    """Parse <action> and return commands"""
+    dom = parseString(action_str)
+    action_node = dom.childNodes[0]
+    delete = Delete(action_node)
+    for cmd in delete.get_commands():
+        yield cmd
+
+
+def _action_str_to_results(action_str):
+    """Parse <action> and return list of results"""
+    return [cmd.execute(False).next() for cmd in _action_str_to_commands(action_str)]
+
+
+def benchmark_regex():
+    """Measure how fast the regex option is"""
+    n_files = 100000
+    print 'benchmark of %d files' % n_files
+
+    # make a directory with many files
+    dirname = tempfile.mkdtemp(prefix='bleachbit-action-bench')
+    for x in xrange(0, n_files):
+        common.touch_file(os.path.join(dirname, str(x)))
+
+    # scan directory
+    import time
+    start = time.time()
+    action_str = '<action command="delete" search="glob" path="%s/*" regex="^12$"/>' % dirname
+    results = _action_str_to_results(action_str)
+    end = time.time()
+    elapsed_seconds = end - start
+    rate = n_files / elapsed_seconds
+    print 'elapsed: %.2f seconds, %.2f files/second' % (elapsed_seconds, rate)
+
+    # clean up
+    shutil.rmtree(dirname)
+
+    return rate
+
+
 def dir_is_empty(dirname):
     """Check whether a directory is empty"""
     return not os.listdir(dirname)
@@ -43,20 +84,6 @@ class ActionTestCase(unittest.TestCase):
 
     """Test cases for Action"""
 
-    def _action_str_to_commands(self, action_str):
-        """Parse <action> and return commands"""
-        dom = parseString(action_str)
-        action_node = dom.childNodes[0]
-        delete = Delete(action_node)
-        for cmd in delete.get_commands():
-            yield cmd
-
-    def _action_str_to_result(self, action_str):
-        """Parse <action> and return result"""
-        cmd = self._action_str_to_commands(action_str).next()
-        result = cmd.execute(False).next()
-        return result
-
     def _test_action_str(self, action_str):
         """Parse <action> and test it"""
         dom = parseString(action_str)
@@ -118,10 +145,13 @@ class ActionTestCase(unittest.TestCase):
                         command = 'delete'
                         filename = filename.replace('\\', '/')
                     else:
+                        # test not needed on this OS
+                        os.remove(filename)
                         continue
                 action_str = '<action command="%s" search="file" path="%s" />' % \
                     (command, filename)
                 self._test_action_str(action_str)
+                self.assert_(not os.path.exists(filename))
 
     def test_ini(self):
         """Unit test for class Ini"""
@@ -151,21 +181,48 @@ class ActionTestCase(unittest.TestCase):
     def test_regex(self):
         """Unit test for regex option"""
         _iglob = glob.iglob
-        glob.iglob = lambda x: ['/tmp/foo1', '/tmp/foo2']
+        glob.iglob = lambda x: ['/tmp/foo1', '/tmp/foo2', '/tmp/bar1']
         _getsize = FileUtilities.getsize
         FileUtilities.getsize = lambda x: 1
-        # return regex match
-        action_str = '<action command="delete" search="glob" path="/tmp/foo" regex="^foo2$"/>'
-        result = self._action_str_to_result(action_str)
-        self.assert_(result['path'], '/tmp/foo2')
-        # return nothing
-        action_str = '<action command="delete" search="glob" path="/tmp/foo" regex="^bar$"/>'
+
+        # should match three files using no regexes
+        action_str = '<action command="delete" search="glob" path="/tmp/foo*" />'
+        results = _action_str_to_results(action_str)
+        self.assert_(3 == len(results))
+
+        # should match second file using positive regex
+        action_str = '<action command="delete" search="glob" path="/tmp/foo*" regex="^foo2$"/>'
+        results = _action_str_to_results(action_str)
+        self.assert_(1 == len(results))
+        self.assertEqual(results[0]['path'], '/tmp/foo2')
+
+        # should match second file using negative regex
+        action_str = '<action command="delete" search="glob" path="/tmp/foo*" nregex="^(foo1|bar1)$"/>'
+        results = _action_str_to_results(action_str)
+        self.assert_(1 == len(results))
+        self.assertEqual(results[0]['path'], '/tmp/foo2')
+
+        # should match second file using both regexes
+        action_str = '<action command="delete" search="glob" path="/tmp/foo*" regex="^f" nregex="1$"/>'
+        results = _action_str_to_results(action_str)
+        self.assert_(1 == len(results))
+        self.assertEqual(results[0]['path'], '/tmp/foo2')
+
+        # should match nothing using positive regex
+        action_str = '<action command="delete" search="glob" path="/tmp/foo*" regex="^bar$"/>'
+        results = _action_str_to_results(action_str)
+        self.assert_(0 == len(results))
+
+        # should match nothing using negative regex
+        action_str = '<action command="delete" search="glob" path="/tmp/foo*" nregex="."/>'
+        results = _action_str_to_results(action_str)
+        self.assert_(0 == len(results))
+
+        # should give an error
+        action_str = '<action command="delete" search="invalid" path="/tmp/foo*" regex="^bar$"/>'
         self.assertRaises(
-            StopIteration, lambda: self._action_str_to_result(action_str))
-        # expect error
-        action_str = '<action command="delete" search="invalid" path="/tmp/foo" regex="^bar$"/>'
-        self.assertRaises(
-            RuntimeError, lambda: self._action_str_to_result(action_str))
+            RuntimeError, lambda: _action_str_to_results(action_str))
+
         # clean up
         glob.iglob = _iglob
         FileUtilities.getsize = _getsize
@@ -195,7 +252,7 @@ class ActionTestCase(unittest.TestCase):
             path = '$WINDIR\\system32'
         action_str = '<action command="delete" search="walk.files" path="%s" />' % path
         results = 0
-        for cmd in self._action_str_to_commands(action_str):
+        for cmd in _action_str_to_commands(action_str):
             result = cmd.execute(False).next()
             common.validate_result(self, result)
             path = result['path']
@@ -210,4 +267,15 @@ def suite():
 
 
 if __name__ == '__main__':
+    if 1 < len(sys.argv) and 'benchmark' == sys.argv[1]:
+        rates = []
+        iterations = 1
+        if 3 == len(sys.argv):
+            iterations = int(sys.argv[2])
+        for x in xrange(0, iterations):
+            rate = benchmark_regex()
+            rates.append(rate)
+        # combine all the rates for easy copy and paste into R for analysis
+        print 'rates=%s' % ','.join([str(rate) for rate in rates])
+        sys.exit()
     unittest.main()
