@@ -34,6 +34,8 @@ import tempfile
 import time
 import unittest
 
+import common
+
 sys.path.append('.')
 from bleachbit.FileUtilities import *
 from bleachbit.Options import options
@@ -131,10 +133,6 @@ class FileUtilitiesTestCase(unittest.TestCase):
 
     """Test case for module FileUtilities"""
 
-    def __touch(self, filename):
-        """Create an empty file"""
-        open(filename, "w")
-
     def test_bytes_to_human(self):
         """Unit test for class bytes_to_human"""
 
@@ -143,19 +141,35 @@ class FileUtilitiesTestCase(unittest.TestCase):
             locale.setlocale(locale.LC_NUMERIC, 'en_US.UTF-8')
 
         # test one-way conversion for predefined values
-        tests = [("-1B", bytes_to_human(-1)),
-                 ("0", bytes_to_human(0)),
-                 ("1B", bytes_to_human(1)),
-                 ("1kB", bytes_to_human(1000)),
-                 ("1.1kB", bytes_to_human(1110)),
-                 ("1MB", bytes_to_human(1000 ** 2)),
-                 ("1.3MB", bytes_to_human(1289748)),
-                 ("1GB", bytes_to_human(1000 ** 3)),
-                 ("1.32GB", bytes_to_human(1320702444)),
-                 ("1TB", bytes_to_human(1000 ** 4))]
+        # each test is a tuple in the format: (bytes, SI, EIC)
+        tests = ((-1, '-1B', '-1B'),
+                 (0, '0', '0'),
+                 (1, '1B', '1B'),
+                 (1000, '1kB', '1000B'),
+                 (1024, '1kB', '1KiB'),
+                 (1110, '1.1kB', '1.1KiB'),
+                 (1000**2, '1MB', '976.6KiB'),
+                 (1024**2, '1MB', '1MiB'),
+                 (1289748, '1.3MB', '1.2MiB'),
+                 (1000**3, '1GB', '953.7MiB'),
+                 (1024**3, '1.07GB', '1GiB'),
+                 (1320702444, '1.32GB', '1.23GiB'),
+                 (1000**4, '1TB', '931.32GiB'),
+                 (1024**4, '1.1TB', '1TiB'),
+                 (1000**5, '1PB', '909.49TiB'),
+                 (1024**5, '1.13PB', '1PiB'))
 
+        options.set('units_iec', True)
         for test in tests:
-            self.assertEqual(test[0], test[1])
+            iec = bytes_to_human(test[0])
+            self.assertEqual(test[2], iec,
+                             'bytes_to_human(%d) IEC = %s but expected %s' % (test[0], iec, test[2]))
+
+        options.set('units_iec', False)
+        for test in tests:
+            si = bytes_to_human(test[0])
+            self.assertEqual(test[1], si,
+                             'bytes_to_human(%d) SI = %s but expected %s' % (test[0], si, test[1]))
 
         # test roundtrip conversion for random values
         import random
@@ -197,7 +211,7 @@ class FileUtilitiesTestCase(unittest.TestCase):
         # test a constructed file in a constructed directory
         dirname = tempfile.mkdtemp(prefix='bleachbit-test-children')
         filename = os.path.join(dirname, "somefile")
-        self.__touch(filename)
+        common.touch_file(filename)
         for loopfilename in children_in_directory(dirname, True):
             self.assertEqual(loopfilename, filename)
         for loopfilename in children_in_directory(dirname, False):
@@ -427,6 +441,24 @@ class FileUtilitiesTestCase(unittest.TestCase):
         self.assert_(result > -1)
         self.assert_(isinstance(result, (int, long)))
 
+        # compare to WMIC
+        if 'nt' != os.name:
+            return
+        args = ['wmic',  'LogicalDisk', 'get', 'DeviceID,', 'FreeSpace']
+        from bleachbit.General import run_external
+        (rc, stdout, stderr) = run_external(args)
+        if rc:
+            print 'error calling WMIC\nargs=%s\nstderr=%s' % (args, stderr)
+            return
+        import re
+        for line in stdout.split('\n'):
+            line = line.strip()
+            if not re.match('([A-Z]):\s+(\d+)', line):
+                continue
+            drive, bytes_free = re.split('\s+', line)
+            bytes_free = int(bytes_free)
+            self.assertEqual(bytes_free, free_space(drive))
+
     def test_getsize(self):
         """Unit test for method getsize()"""
         dirname = tempfile.mkdtemp(prefix='bleachbit-test-getsize')
@@ -514,7 +546,7 @@ class FileUtilitiesTestCase(unittest.TestCase):
     def test_guess_overwrite_paths(self):
         """Unit test for guess_overwrite_paths()"""
         for path in guess_overwrite_paths():
-            self.assert_(os.path.isdir(path))
+            self.assert_(os.path.isdir(path), '%s is not a directory' % path)
 
     def test_listdir(self):
         """Unit test for listdir()"""
