@@ -28,6 +28,7 @@ import sys
 import tempfile
 import unittest
 import platform
+from decimal import Decimal
 
 import common
 
@@ -203,6 +204,13 @@ class WindowsTestCase(unittest.TestCase):
             self.assertEqual(drive, drive.upper())
         self.assert_("C:\\" in drives)
 
+    def test_get_windows_version(self):
+        """Unit test for get_windows_version"""
+        v = get_windows_version()
+        self.assert_(v >= 5.1)
+        self.assert_(v > 5)
+        self.assert_(isinstance(v, Decimal))
+
     def test_empty_recycle_bin(self):
         """Unit test for empty_recycle_bin"""
         # check the function basically works
@@ -229,14 +237,14 @@ class WindowsTestCase(unittest.TestCase):
             self.assertEqual(ret, 0)
 
     def test_is_process_running(self):
-        tests = ((True, 'explorer.exe'),
-                 (True, 'ExPlOrEr.exe'),
+        # winlogon.exe runs on Windows XP and Windows 7
+        # explorer.exe does not run on Appveyor
+        tests = ((True, 'winlogon.exe'),
+                 (True, 'WinLogOn.exe'),
                  (False, 'doesnotexist.exe'))
         for test in tests:
             self.assertEqual(test[0], is_process_running(
-                test[1]), 'is_process_running(%s) != %s' % (test[1], test[0]))
-            self.assertEqual(test[0], is_process_running_win32(test[1]))
-            self.assertEqual(test[0], is_process_running_wmic(test[1]))
+                test[1]), 'Expecting is_process_running(%s) = %s' % (test[1], test[0]))
 
     def test_setup_environment(self):
         """Unit test for setup_environment"""
@@ -274,18 +282,32 @@ class WindowsTestCase(unittest.TestCase):
             self.assert_(isinstance(b, bool))
             self.assertEqual(b, three_b)
 
+    def test_parse_windows_build(self):
+        """Unit test for parse_windows_build"""
+        tests = (('5.1.2600', Decimal('5.1')),
+                 ('5.1', Decimal('5.1')),
+                 ('10.0.10240', 10),
+                 ('10.0', 10))
+        for test in tests:
+            self.assertEqual(parse_windows_build(test[0]), test[1])
+
+        # test for crash
+        parse_windows_build()
+        parse_windows_build(platform.version())
+        parse_windows_build(platform.uname()[3])
+
     def test_path_on_network(self):
         """Unit test for path_on_network"""
         self.assertFalse(path_on_network('c:\\bleachbit.exe'))
         self.assertFalse(path_on_network('a:\\bleachbit.exe'))
         self.assertTrue(path_on_network('\\\\Server\\Folder\\bleachbit.exe'))
 
-
     def test_shell_change_notify(self):
         """Unit test for shell_change_notify"""
         ret = shell_change_notify()
         self.assertEqual(ret, 0)
 
+
 def suite():
     return unittest.makeSuite(WindowsTestCase)
 
