@@ -35,6 +35,12 @@ def destructive_tests(title):
     return False
 
 
+def touch_file(filename):
+    """Create an empty file"""
+    f = open(filename, "w")
+    f.close()
+
+
 def validate_result(self, result, really_delete=False):
     """Validate the command returned valid results"""
     self.assert_(isinstance(result, dict), "result is a %s" % type(result))
